package org.apache.bcel.classfile;

import java.util.Arrays;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public class BootstrapMethod implements Cloneable
{
    private int bootstrapMethodRef;
    private int[] bootstrapArguments;
    
    public BootstrapMethod(final BootstrapMethod c) {
        this(c.getBootstrapMethodRef(), c.getBootstrapArguments());
    }
    
    BootstrapMethod(final DataInput input) throws IOException {
        this(input.readUnsignedShort(), input.readUnsignedShort());
        for (int i = 0; i < this.bootstrapArguments.length; ++i) {
            this.bootstrapArguments[i] = input.readUnsignedShort();
        }
    }
    
    private BootstrapMethod(final int bootstrapMethodRef, final int numBootstrapArguments) {
        this(bootstrapMethodRef, new int[numBootstrapArguments]);
    }
    
    public BootstrapMethod(final int bootstrapMethodRef, final int[] bootstrapArguments) {
        this.bootstrapMethodRef = bootstrapMethodRef;
        this.bootstrapArguments = bootstrapArguments;
    }
    
    public BootstrapMethod copy() {
        try {
            return (BootstrapMethod)this.clone();
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public final void dump(final DataOutputStream file) throws IOException {
        file.writeShort(this.bootstrapMethodRef);
        file.writeShort(this.bootstrapArguments.length);
        for (final int bootstrapArgument : this.bootstrapArguments) {
            file.writeShort(bootstrapArgument);
        }
    }
    
    public int[] getBootstrapArguments() {
        return this.bootstrapArguments;
    }
    
    public int getBootstrapMethodRef() {
        return this.bootstrapMethodRef;
    }
    
    public int getNumBootstrapArguments() {
        return this.bootstrapArguments.length;
    }
    
    public void setBootstrapArguments(final int[] bootstrapArguments) {
        this.bootstrapArguments = bootstrapArguments;
    }
    
    public void setBootstrapMethodRef(final int bootstrapMethodRef) {
        this.bootstrapMethodRef = bootstrapMethodRef;
    }
    
    @Override
    public final String toString() {
        return "BootstrapMethod(" + this.bootstrapMethodRef + ", " + this.bootstrapArguments.length + ", " + Arrays.toString(this.bootstrapArguments) + ")";
    }
    
    public final String toString(final ConstantPool constantPool) {
        final StringBuilder buf = new StringBuilder();
        final String bootstrapMethodName = constantPool.constantToString(this.bootstrapMethodRef, (byte)15);
        buf.append(Utility.compactClassName(bootstrapMethodName, false));
        final int bootstrapArgumentsLen = this.bootstrapArguments.length;
        if (bootstrapArgumentsLen > 0) {
            buf.append("\nMethod Arguments:");
            for (int i = 0; i < bootstrapArgumentsLen; ++i) {
                buf.append("\n  ").append(i).append(": ");
                buf.append(constantPool.constantToString(constantPool.getConstant(this.bootstrapArguments[i])));
            }
        }
        return buf.toString();
    }
}
